@@ -1,4 +1,5 @@
-// Last edit: 2/10/07
+/* $Id: CoinLpIO.cpp 1448 2011-06-19 15:34:41Z stefan $ */
+// Last edit: 11/5/08
 //
 // Name:     CoinLpIO.cpp; Support for Lp files
 // Author:   Francois Margot
@@ -8,14 +9,22 @@
 //-----------------------------------------------------------------------------
 // Copyright (C) 2003, Francois Margot, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
+#include "CoinUtilsConfig.h"
+
+#include <cmath>
 #include <cfloat>
 #include <cctype>
 #include <cassert>
+#include <string>
+#include <cstdarg>
 
+#include "CoinError.hpp"
 #include "CoinHelperFunctions.hpp"
 #include "CoinPackedMatrix.hpp"
 #include "CoinLpIO.hpp"
+#include "CoinFinite.hpp"
 
 using namespace std;
 
@@ -25,6 +34,7 @@ using namespace std;
 
 CoinLpIO::CoinLpIO() :
   problemName_(CoinStrdup("")),
+  defaultHandler_(true),
   numberRows_(0),
   numberColumns_(0),
   numberElements_(0),
@@ -41,7 +51,7 @@ CoinLpIO::CoinLpIO() :
   objectiveOffset_(0),
   integerType_(NULL),
   fileName_(NULL),
-  infinity_(DBL_MAX),
+  infinity_(COIN_DBL_MAX),
   epsilon_(1e-5),
   numberAcross_(10),
   decimals_(5),
@@ -60,6 +70,8 @@ CoinLpIO::CoinLpIO() :
   numberHash_[1] = 0;
   hash_[1] = NULL;
   names_[1] = NULL;
+  handler_ = new CoinMessageHandler();
+  messages_ = CoinMessage();
 }
 
 /************************************************************************/
@@ -67,6 +79,9 @@ CoinLpIO::~CoinLpIO() {
   stopHash(0);
   stopHash(1);
   freeAll();
+  if (defaultHandler_) {
+    delete handler_;
+  }
 }
 
 /************************************************************************/
@@ -211,7 +226,7 @@ CoinLpIO::convertBoundToSense(const double lower, const double upper,
 {
   if(rowsense_  == NULL) {
     int nr=numberRows_;
-    rowsense_ = (char *) malloc(nr*sizeof(char));
+    rowsense_ = reinterpret_cast<char *> (malloc(nr*sizeof(char)));
     
     double dum1,dum2;
     int i;
@@ -227,7 +242,7 @@ const double * CoinLpIO::getRightHandSide() const
 {
   if(rhs_==NULL) {
     int nr=numberRows_;
-    rhs_ = (double *) malloc(nr*sizeof(double));
+    rhs_ = reinterpret_cast<double *> (malloc(nr*sizeof(double)));
 
     char dum1;
     double dum2;
@@ -244,7 +259,7 @@ const double * CoinLpIO::getRowRange() const
 {
   if (rowrange_ == NULL) {
     int nr=numberRows_;
-    rowrange_ = (double *) malloc(nr*sizeof(double));
+    rowrange_ = reinterpret_cast<double *> (malloc(nr*sizeof(double)));
     std::fill(rowrange_,rowrange_+nr,0.0);
 
     char dum1;
@@ -292,7 +307,9 @@ void CoinLpIO::checkRowNames() {
 
   if(numberHash_[0] != nrow+1) {
     setDefaultRowNames();
-    printf("### WARNING: CoinLpIO::checkRowNames(): non distinct or missing row names or objective function name.\nNow using default row names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::checkRowNames(): non distinct or missing row names or objective function name.\nNow using default row names."
+						     <<CoinMessageEol;
   }
 
   char const * const * rowNames = getRowNames();
@@ -307,7 +324,10 @@ void CoinLpIO::checkRowNames() {
       sprintf(rName, "%s_low", rowNames[i]);
       if(findHash(rName, 0) != -1) {
 	setDefaultRowNames();
-	printf("### WARNING: CoinLpIO::checkRowNames(): ranged constraint %d hasa name %s identical to another constraint name or objective function name.\nUse getPreviousNames() to get the old row names.\nNow using default row names.\n", i, rName);
+	char printBuffer[512];
+	sprintf(printBuffer,"### CoinLpIO::checkRowNames(): ranged constraint %d hasa name %s identical to another constraint name or objective function name.\nUse getPreviousNames() to get the old row names.\nNow using default row names.", i, rName);
+	handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						  <<CoinMessageEol;
 	break;
       }
     }
@@ -321,7 +341,9 @@ void CoinLpIO::checkColNames() {
 
   if(numberHash_[1] != ncol) {
     setDefaultColNames();
-    printf("### WARNING: CoinLpIO::checkColNames(): non distinct or missing column names.\nNow using default column names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::checkColNames(): non distinct or missing column names.\nNow using default column names."
+					      <<CoinMessageEol;
   }
 } /* checkColNames */
 
@@ -402,12 +424,13 @@ double CoinLpIO::getInfinity() const
 /************************************************************************/
 void CoinLpIO::setInfinity(const double value) 
 {
-  if (value >= 1.020) {
+  if (value >= 1.0e20) {
     infinity_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setInfinity(): value: %f\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %f\n", value);
+    throw CoinError(str, "setInfinity", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -424,8 +447,9 @@ void CoinLpIO::setEpsilon(const double value)
     epsilon_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setEpsilon(): value: %f\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %f\n", value);
+    throw CoinError(str, "setEpsilon", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -442,8 +466,9 @@ void CoinLpIO::setNumberAcross(const int value)
     numberAcross_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setNumberAcross(): value: %d\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %d\n", value);
+    throw CoinError(str, "setNumberAcross", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -460,8 +485,9 @@ void CoinLpIO::setDecimals(const int value)
     decimals_ = value;
   } 
   else {
-    printf("### ERROR: CoinLpIO::setDecimals(): value: %d\n", value);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: value: %d\n", value);
+    throw CoinError(str, "setDecimals", "CoinLpIO", __FILE__, __LINE__);
   }
 }
 
@@ -509,11 +535,11 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
   numberColumns_ = matrixByRow_->getNumCols();
   numberRows_ = matrixByRow_->getNumRows();
   
-  rowlower_ = (double *) malloc (numberRows_ * sizeof(double));
-  rowupper_ = (double *) malloc (numberRows_ * sizeof(double));
-  collower_ = (double *) malloc (numberColumns_ * sizeof(double));
-  colupper_ = (double *) malloc (numberColumns_ * sizeof(double));
-  objective_ = (double *) malloc (numberColumns_ * sizeof(double));
+  rowlower_ = reinterpret_cast<double *> (malloc (numberRows_ * sizeof(double)));
+  rowupper_ = reinterpret_cast<double *> (malloc (numberRows_ * sizeof(double)));
+  collower_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+  colupper_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
+  objective_ = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   std::copy(rowlb, rowlb + numberRows_, rowlower_);
   std::copy(rowub, rowub + numberRows_, rowupper_);
   std::copy(collb, collb + numberColumns_, collower_);
@@ -521,7 +547,7 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
   std::copy(obj_coeff, obj_coeff + numberColumns_, objective_);
 
   if (is_integer) {
-    integerType_ = (char *) malloc (numberColumns_ * sizeof(char));
+    integerType_ = reinterpret_cast<char *> (malloc (numberColumns_ * sizeof(char)));
     std::copy(is_integer, is_integer + numberColumns_, integerType_);
   } 
   else {
@@ -540,7 +566,7 @@ CoinLpIO::setLpDataWithoutRowAndColNames(
 void CoinLpIO::setDefaultRowNames() {
 
   int i, nrow = getNumRows();
-  char **defaultRowNames = (char **) malloc ((nrow+1) * sizeof(char *));
+  char **defaultRowNames = reinterpret_cast<char **> (malloc ((nrow+1) * sizeof(char *)));
   char buff[1024];
 
   for(i=0; i<nrow; i++) {
@@ -565,7 +591,7 @@ void CoinLpIO::setDefaultRowNames() {
 void CoinLpIO::setDefaultColNames() {
 
   int j, ncol = getNumCols();
-  char **defaultColNames = (char **) malloc (ncol * sizeof(char *));
+  char **defaultColNames = reinterpret_cast<char **> (malloc (ncol * sizeof(char *)));
   char buff[256];
 
   for(j=0; j<ncol; j++) {
@@ -592,7 +618,9 @@ void CoinLpIO::setLpDataRowAndColNames(char const * const * const rownames,
   if(rownames != NULL) {
     if(are_invalid_names(rownames, nrow+1, true)) {
       setDefaultRowNames();
-      printf("### WARNING: CoinLpIO::setLpDataRowAndColNames(): Invalid row names\nUse getPreviousNames() to get the old row names.\nNow using default row names.\n");
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::setLpDataRowAndColNames(): Invalid row names\nUse getPreviousNames() to get the old row names.\nNow using default row names."
+						<<CoinMessageEol;
     } 
     else {
       stopHash(0);
@@ -610,7 +638,9 @@ void CoinLpIO::setLpDataRowAndColNames(char const * const * const rownames,
   if(colnames != NULL) {
     if(are_invalid_names(colnames, ncol, false)) {
       setDefaultColNames();
-      printf("### WARNING: CoinLpIO::setLpDataRowAndColNames(): Invalid column names\nNow using default row names.\n");
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::setLpDataRowAndColNames(): Invalid column names\nNow using default row names."
+						<<CoinMessageEol;
     } 
     else {
       stopHash(1);
@@ -660,16 +690,16 @@ CoinLpIO::writeLp(const char *filename, const double epsilon,
 		  const int numberAcross, const int decimals,
 		  const bool useRowNames) {
 
-   FILE *fp = NULL;
-   fp = fopen(filename,"w");
-   if (!fp) {
-     printf("### ERROR: in CoinLpIO::writeLP(): unable to open file %s\n",
-	    filename);
-     exit(1);
-   }
-   int nerr = writeLp(fp, epsilon, numberAcross, decimals, useRowNames);
-   fclose(fp);
-   return(nerr);
+  FILE *fp = NULL;
+  fp = fopen(filename,"w");
+  if (!fp) {
+    char str[8192];
+    sprintf(str,"### ERROR: unable to open file %s\n", filename);
+    throw CoinError(str, "writeLP", "CoinLpIO", __FILE__, __LINE__);
+  }
+  int nerr = writeLp(fp, epsilon, numberAcross, decimals, useRowNames);
+  fclose(fp);
+  return(nerr);
 }
 
 /************************************************************************/
@@ -688,16 +718,16 @@ CoinLpIO::writeLp(FILE *fp, const double epsilon,
 int
 CoinLpIO::writeLp(const char *filename, const bool useRowNames)
 {
-   FILE *fp = NULL;
-   fp = fopen(filename,"w");
-   if (!fp) {
-     printf("### ERROR: in CoinLpIO::writeLP(): unable to open file %s\n",
-	    filename);
-     exit(1);
-   }
-   int nerr = writeLp(fp, useRowNames);
-   fclose(fp);
-   return(nerr);
+  FILE *fp = NULL;
+  fp = fopen(filename,"w");
+  if (!fp) {
+    char str[8192];
+    sprintf(str,"### ERROR: unable to open file %s\n", filename);
+    throw CoinError(str, "writeLP", "CoinLpIO", __FILE__, __LINE__);
+  }
+  int nerr = writeLp(fp, useRowNames);
+  fclose(fp);
+  return(nerr);
 }
 
 /************************************************************************/
@@ -728,7 +758,7 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
 
    if(rowNames == NULL) {
      loc_row_names = 1;
-     prowNames = (char **) malloc ((nrow+1) * sizeof(char *));
+     prowNames = reinterpret_cast<char **> (malloc ((nrow+1) * sizeof(char *)));
 
      for (j=0; j<nrow; j++) {
        sprintf(buff, "cons%d", j);
@@ -740,7 +770,7 @@ CoinLpIO::writeLp(FILE *fp, const bool useRowNames)
 
    if(colNames == NULL) {
      loc_col_names = 1;
-     pcolNames = (char **) malloc (ncol * sizeof(char *));
+     pcolNames = reinterpret_cast<char **> (malloc (ncol * sizeof(char *)));
 
      for (j=0; j<ncol; j++) {
        sprintf(buff, "x%d", j);
@@ -949,7 +979,7 @@ CoinLpIO::find_obj(FILE *fp) const {
   char buff[1024];
 
   sprintf(buff, "aa");
-  unsigned lbuff = strlen(buff);
+  size_t lbuff = strlen(buff);
 
   while(((lbuff != 8) || (CoinStrNCaseCmp(buff, "minimize", 8) != 0)) &&
 	((lbuff != 3) || (CoinStrNCaseCmp(buff, "min", 3) != 0)) &&
@@ -958,10 +988,11 @@ CoinLpIO::find_obj(FILE *fp) const {
 
     scan_next(buff, fp);
     lbuff = strlen(buff);
-
+    
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO: find_obj(): Unable to locate objective function\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Unable to locate objective function\n");
+      throw CoinError(str, "find_obj", "CoinLpIO", __FILE__, __LINE__);
     }
   }
 
@@ -976,7 +1007,7 @@ CoinLpIO::find_obj(FILE *fp) const {
 int
 CoinLpIO::is_subject_to(const char *buff) const {
 
-  unsigned lbuff = strlen(buff);
+  size_t lbuff = strlen(buff);
 
   if(((lbuff == 4) && (CoinStrNCaseCmp(buff, "s.t.", 4) == 0)) ||
      ((lbuff == 3) && (CoinStrNCaseCmp(buff, "st.", 3) == 0)) ||
@@ -993,11 +1024,11 @@ CoinLpIO::is_subject_to(const char *buff) const {
 int 
 CoinLpIO::first_is_number(const char *buff) const {
 
-  int pos;
+  size_t pos;
   char str_num[] = "1234567890";
 
   pos = strcspn (buff, str_num);
-  if(pos == 0) {
+  if (pos == 0) {
     return(1);
   }
   return(0);
@@ -1007,7 +1038,7 @@ CoinLpIO::first_is_number(const char *buff) const {
 int 
 CoinLpIO::is_sense(const char *buff) const {
 
-  int pos;
+  size_t pos;
   char str_sense[] = "<>=";
 
   pos = strcspn(buff, str_sense);
@@ -1031,7 +1062,7 @@ CoinLpIO::is_sense(const char *buff) const {
 int 
 CoinLpIO::is_free(const char *buff) const {
 
-  unsigned lbuff = strlen(buff);
+  size_t lbuff = strlen(buff);
 
   if((lbuff == 4) && (CoinStrNCaseCmp(buff, "free", 4) == 0)) {
     return(1);
@@ -1039,6 +1070,18 @@ CoinLpIO::is_free(const char *buff) const {
   return(0);
 } /* is_free */
 
+/*************************************************************************/
+int 
+CoinLpIO::is_inf(const char *buff) const {
+
+  size_t lbuff = strlen(buff);
+
+  if((lbuff == 3) && (CoinStrNCaseCmp(buff, "inf", 3) == 0)) {
+    return(1);
+  }
+  return(0);
+} /* is_inf */
+
 /*************************************************************************/
 int 
 CoinLpIO::is_comment(const char *buff) const {
@@ -1055,14 +1098,18 @@ CoinLpIO::skip_comment(char *buff, FILE *fp) const {
 
   while(strcspn(buff, "\n") == strlen(buff)) { // end of line not read yet
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO::skip_comment(): end of file reached while skipping comment\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: end of file reached while skipping comment\n");
+      throw CoinError(str, "skip_comment", "CoinLpIO", __FILE__, __LINE__);
     }
     if(ferror(fp)) {
-      printf("### ERROR: CoinLpIO::skip_comment(): error while skipping comment\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: error while skipping comment\n");
+      throw CoinError(str, "skip_comment", "CoinLpIO", __FILE__, __LINE__);
     }
-    fgets(buff, sizeof(buff), fp);    
+    char * x=fgets(buff, sizeof(buff), fp);    
+    if (!x)
+      throw("bad fgets");
   } 
 } /* skip_comment */
 
@@ -1070,10 +1117,14 @@ CoinLpIO::skip_comment(char *buff, FILE *fp) const {
 void
 CoinLpIO::scan_next(char *buff, FILE *fp) const {
 
-  fscanf(fp, "%s", buff);
+  int x=fscanf(fp, "%s", buff);
+  if (x<=0)
+    throw("bad fscanf");
   while(is_comment(buff)) {
     skip_comment(buff, fp);
-    fscanf(fp, "%s", buff);
+    x=fscanf(fp, "%s", buff);
+    if (x<=0)
+      throw("bad fscanf");
   }
 
 #ifdef LPIO_DEBUG
@@ -1087,7 +1138,7 @@ int
 CoinLpIO::is_invalid_name(const char *name, 
 			  const bool ranged) const {
 
-  unsigned pos, lname, valid_lname = 100;
+  size_t pos, lname, valid_lname = 100;
   char str_valid[] = "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ\"!#$%&(),.;?@_'`{}~";
 
   if(ranged) {
@@ -1101,25 +1152,36 @@ CoinLpIO::is_invalid_name(const char *name,
     lname = strlen(name);
   }
   if(lname < 1) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name is empty\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+    "### CoinLpIO::is_invalid_name(): Name is empty"
+					      <<CoinMessageEol;
     return(5);
   }
   if(lname > valid_lname) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s is too long\n", 
+	char printBuffer[512];
+	sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s is too long", 
 	   name);
+	handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							 <<CoinMessageEol;
     return(1);
   }
   if(first_is_number(name)) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s should not start with a number\n", name);
+    char printBuffer[512];
+    sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s should not start with a number", name);
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						     <<CoinMessageEol;
     return(2);
   }
   pos = strspn(name, str_valid);
   if(pos != lname) {
-    printf("### WARNING: CoinLpIO::is_invalid_name(): Name %s contains illegal character '%c'\n", name, name[pos]);
+    char printBuffer[512];
+    sprintf(printBuffer,"### CoinLpIO::is_invalid_name(): Name %s contains illegal character '%c'", name, name[pos]);
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						     <<CoinMessageEol;
     return(3);
   }
 
-  if((is_keyword(name)) || (is_free(name))) {
+  if((is_keyword(name)) || (is_free(name) || (is_inf(name)))) {
     return(4);
   }
 
@@ -1137,8 +1199,10 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
   const char * rSense = getRowSense();
 
   if((check_ranged) && (card_vnames != nrows+1)) {
-    printf("### ERROR: CoinLpIO::are_invalid_names(): card_vnames: %d   number of rows: %d\n", card_vnames, getNumRows());
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: card_vnames: %d   number of rows: %d\n",
+	    card_vnames, getNumRows());
+    throw CoinError(str, "are_invalid_names", "CoinLpIO", __FILE__, __LINE__);
   }
 
   for(i=0; i<card_vnames; i++) {
@@ -1151,8 +1215,11 @@ CoinLpIO::are_invalid_names(char const * const * const vnames,
     }
     flag = is_invalid_name(vnames[i], is_ranged);
     if(flag) {
-      printf("### WARNING: CoinLpIO::are_invalid_names(): invalid name: vnames[%d]: %s\n",
-	     i, vnames[i]);
+      char printBuffer[512];
+      sprintf(printBuffer,"### CoinLpIO::are_invalid_names(): Invalid name: vnames[%d]: %s",
+	      i, vnames[i]);
+      handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+						       <<CoinMessageEol;
       invalid = flag;
     }
   }
@@ -1171,8 +1238,9 @@ CoinLpIO::read_monom_obj(FILE *fp, double *coeff, char **name, int *cnt,
   scan_next(buff, fp);
 
   if(feof(fp)) {
-    printf("### ERROR: CoinLpIO:  read_monom_obj(): Unable to read objective function\n");
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: Unable to read objective function\n");
+    throw CoinError(str, "read_monom_obj", "CoinLpIO", __FILE__, __LINE__);
   }
 
   if(buff[strlen(buff)-1] == ':') {
@@ -1269,7 +1337,7 @@ CoinLpIO::read_monom_row(FILE *fp, char *start_str,
   char buff[1024], loc_name[1024], *start;
   int read_sense = -1;
 
-  sprintf(buff, start_str);
+  sprintf(buff, "%s", start_str);
   read_sense = is_sense(buff);
   if(read_sense > -1) {
     return(read_sense);
@@ -1309,7 +1377,14 @@ CoinLpIO::read_monom_row(FILE *fp, char *start_str,
   }
 
   coeff[cnt_coeff] *= mult;
+#ifdef KILL_ZERO_READLP
+  if (fabs(coeff[cnt_coeff])>epsilon_)
+    name[cnt_coeff] = CoinStrdup(loc_name);
+  else
+    read_sense=-2; // effectively zero
+#else
   name[cnt_coeff] = CoinStrdup(loc_name);
+#endif
 
 #ifdef LPIO_DEBUG
   printf("CoinLpIO: read_monom_row: (%f)  (%s)\n", 
@@ -1325,8 +1400,8 @@ CoinLpIO::realloc_coeff(double **coeff, char ***colNames,
   
   *maxcoeff *= 5;
 
-  *colNames = (char **) realloc ((*colNames), (*maxcoeff+1) * sizeof(char *));
-  *coeff = (double *) realloc ((*coeff), (*maxcoeff+1) * sizeof(double));
+  *colNames = reinterpret_cast<char **> (realloc ((*colNames), (*maxcoeff+1) * sizeof(char *)));
+  *coeff = reinterpret_cast<double *> (realloc ((*coeff), (*maxcoeff+1) * sizeof(double)));
 
 } /* realloc_coeff */
 
@@ -1336,11 +1411,11 @@ CoinLpIO::realloc_row(char ***rowNames, int **start, double **rhs,
 		      double **rowlow, double **rowup, int *maxrow) const {
 
   *maxrow *= 5;
-  *rowNames = (char **) realloc ((*rowNames), (*maxrow+1) * sizeof(char *));
-  *start = (int *) realloc ((*start), (*maxrow+1) * sizeof(int));
-  *rhs = (double *) realloc ((*rhs), (*maxrow+1) * sizeof(double));
-  *rowlow = (double *) realloc ((*rowlow), (*maxrow+1) * sizeof(double));
-  *rowup = (double *) realloc ((*rowup), (*maxrow+1) * sizeof(double));
+  *rowNames = reinterpret_cast<char **> (realloc ((*rowNames), (*maxrow+1) * sizeof(char *)));
+  *start = reinterpret_cast<int *> (realloc ((*start), (*maxrow+1) * sizeof(int)));
+  *rhs = reinterpret_cast<double *> (realloc ((*rhs), (*maxrow+1) * sizeof(double)));
+  *rowlow = reinterpret_cast<double *> (realloc ((*rowlow), (*maxrow+1) * sizeof(double)));
+  *rowup = reinterpret_cast<double *> (realloc ((*rowup), (*maxrow+1) * sizeof(double)));
 
 } /* realloc_row */
 
@@ -1350,9 +1425,9 @@ CoinLpIO::realloc_col(double **collow, double **colup, char **is_int,
 		      int *maxcol) const {
   
   *maxcol += 100;
-  *collow = (double *) realloc ((*collow), (*maxcol+1) * sizeof(double));
-  *colup = (double *) realloc ((*colup), (*maxcol+1) * sizeof(double));
-  *is_int = (char *) realloc ((*is_int), (*maxcol+1) * sizeof(char));
+  *collow = reinterpret_cast<double *> (realloc ((*collow), (*maxcol+1) * sizeof(double)));
+  *colup = reinterpret_cast<double *> (realloc ((*colup), (*maxcol+1) * sizeof(double)));
+  *is_int = reinterpret_cast<char *> (realloc ((*is_int), (*maxcol+1) * sizeof(char)));
 
 } /* realloc_col */
 
@@ -1368,7 +1443,7 @@ CoinLpIO::read_row(FILE *fp, char *buff,
   int read_sense = -1;
   char start_str[1024];
   
-  sprintf(start_str, buff);
+  sprintf(start_str, "%s", buff);
 
   while(read_sense < 0) {
 
@@ -1377,14 +1452,17 @@ CoinLpIO::read_row(FILE *fp, char *buff,
     }
     read_sense = read_monom_row(fp, start_str, 
 				*pcoeff, *pcolNames, *cnt_coeff);
-
-    (*cnt_coeff)++;
+#ifdef KILL_ZERO_READLP
+    if (read_sense!=-2) // see if zero
+#endif
+      (*cnt_coeff)++;
 
     scan_next(start_str, fp);
 
     if(feof(fp)) {
-      printf("### ERROR: CoinLpIO:  read_monom_row(): Unable to read row monomial\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Unable to read row monomial\n");
+      throw CoinError(str, "read_monom_row", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   (*cnt_coeff)--;
@@ -1408,7 +1486,7 @@ CoinLpIO::read_row(FILE *fp, char *buff,
 int 
 CoinLpIO::is_keyword(const char *buff) const {
 
-  unsigned lbuff = strlen(buff);
+  size_t lbuff = strlen(buff);
 
   if(((lbuff == 5) && (CoinStrNCaseCmp(buff, "bound", 5) == 0)) ||
      ((lbuff == 6) && (CoinStrNCaseCmp(buff, "bounds", 6) == 0))) {
@@ -1452,9 +1530,9 @@ CoinLpIO::readLp(const char *filename)
 {
   FILE *fp = fopen(filename, "r");
   if(!fp) {
-    printf("### ERROR: CoinLpIO:  Unable to open file %s for reading\n",
-	   filename);
-    exit(1);
+    char str[8192];
+    sprintf(str,"### ERROR: Unable to open file %s for reading\n", filename);
+    throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
   }
   readLp(fp);
   fclose(fp);
@@ -1483,14 +1561,14 @@ CoinLpIO::readLp(FILE* fp)
   int objsense, cnt_coeff = 0, cnt_row = 0, cnt_obj = 0;
   char *objName = NULL;
 
-  char **colNames = (char **) malloc ((maxcoeff+1) * sizeof(char *));
-  double *coeff = (double *) malloc ((maxcoeff+1) * sizeof(double));
+  char **colNames = reinterpret_cast<char **> (malloc ((maxcoeff+1) * sizeof(char *)));
+  double *coeff = reinterpret_cast<double *> (malloc ((maxcoeff+1) * sizeof(double)));
 
-  char **rowNames = (char **) malloc ((maxrow+1) * sizeof(char *));
-  int *start = (int *) malloc ((maxrow+1) * sizeof(int));
-  double *rhs = (double *) malloc ((maxrow+1) * sizeof(double));
-  double *rowlow = (double *) malloc ((maxrow+1) * sizeof(double));
-  double *rowup = (double *) malloc ((maxrow+1) * sizeof(double));
+  char **rowNames = reinterpret_cast<char **> (malloc ((maxrow+1) * sizeof(char *)));
+  int *start = reinterpret_cast<int *> (malloc ((maxrow+1) * sizeof(int)));
+  double *rhs = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
+  double *rowlow = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
+  double *rowup = reinterpret_cast<double *> (malloc ((maxrow+1) * sizeof(double)));
 
   int i;
 
@@ -1509,12 +1587,15 @@ CoinLpIO::readLp(FILE* fp)
   cnt_coeff = cnt_obj;
 
   if(read_st == 2) {
-    fscanf(fp, "%s", buff);
-    unsigned lbuff = strlen(buff);
+    int x=fscanf(fp, "%s", buff);
+    if (x<=0)
+      throw("bad fscanf");
+    size_t lbuff = strlen(buff);
 
     if((lbuff != 2) || (CoinStrNCaseCmp(buff, "to", 2) != 0)) {
-      printf("### ERROR: CoinLpIO::readLp(): Can not locate keyword 'Subject To'\n");
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Can not locate keyword 'Subject To'\n");
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   
@@ -1559,9 +1640,9 @@ CoinLpIO::readLp(FILE* fp)
 
   int maxcol = numberHash_[1] + 100;
 
-  double *collow = (double *) malloc ((maxcol+1) * sizeof(double));
-  double *colup = (double *) malloc ((maxcol+1) * sizeof(double));
-  char *is_int = (char *) malloc ((maxcol+1) * sizeof(char));
+  double *collow = reinterpret_cast<double *> (malloc ((maxcol+1) * sizeof(double)));
+  double *colup = reinterpret_cast<double *> (malloc ((maxcol+1) * sizeof(double)));
+  char *is_int = reinterpret_cast<char *> (malloc ((maxcol+1) * sizeof(char)));
   int has_int = 0;
 
   for (i=0; i<maxcol; i++) {
@@ -1585,8 +1666,8 @@ CoinLpIO::readLp(FILE* fp)
 	int mult = 1;
 	char *start_str = buff;
 
-	if(buff[0] == '-') {
-	  mult = -1;
+	if(buff[0] == '-' || buff[0] == '+') {
+	  mult = (buff[0] == '-') ? -1 : +1;
 	  if(strlen(buff) == 1) {
 	    scan_next(buff, fp);
 	    start_str = buff;
@@ -1596,20 +1677,34 @@ CoinLpIO::readLp(FILE* fp)
 	  }
 	}
 
+	int scan_sense = 0;
 	if(first_is_number(start_str)) {
 	  bnd1 = mult * atof(start_str);
+	  scan_sense = 1;
+	}
+	else {
+	  if(is_inf(start_str)) {
+	    bnd1 = mult * lp_inf;
+	    scan_sense = 1;
+	  }
+	}
+	if(scan_sense) {
 	  scan_next(buff, fp);
 	  read_sense1 = is_sense(buff);
 	  if(read_sense1 < 0) {
-	    printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; expect a sense, get: %s\n", buff);
-	    exit(1);
+	    char str[8192];
+	    sprintf(str,"### ERROR: Bounds; expect a sense, get: %s\n", buff);
+	    throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 	  }
 	  scan_next(buff, fp);
 	}
 
 	icol = findHash(buff, 1);
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1644,29 +1739,33 @@ CoinLpIO::readLp(FILE* fp)
 	      scan_next(buff, fp);
 	    }
 	    else {
-	      if (strncmp(start_str,"inf",3)&&strncmp(start_str,"Inf",3)) {
-		printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; expect a number, get: %s\n", buff);
-		exit(1);
-	      } else {
-		// infinity
-		bnd2 = DBL_MAX;
+	      if(is_inf(start_str)) {
+		bnd2 = mult * lp_inf;
 		scan_next(buff, fp);
 	      }
+	      else {
+		char str[8192];
+		sprintf(str,"### ERROR: Bounds; expect a number, get: %s\n",
+			buff);
+		throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
+	      }
 	    }
 	  }
 
 	  if((read_sense1 > -1) && (read_sense2 > -1)) {
 	    if(read_sense1 != read_sense2) {
-	      printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; \n\
-variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2);
-	      exit(1);
+	      char str[8192];
+	      sprintf(str,"### ERROR: Bounds; variable: %s read_sense1: %d  read_sense2: %d\n",
+		      buff, read_sense1, read_sense2);
+	      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 	    }
 	    else {
 	      if(read_sense1 == 1) {
 		if(fabs(bnd1 - bnd2) > lp_eps) {
-		  printf("### ERROR: CoinLpIO::readLp(): lost in Bounds section; \nvariable: %s read_sense1: %d  read_sense2: %d  bnd1: %f  bnd2: %f\n", 
-			 buff, read_sense1, read_sense2, bnd1, bnd2);
-		  exit(1);
+		  char str[8192];
+		  sprintf(str,"### ERROR: Bounds; variable: %s read_sense1: %d  read_sense2: %d  bnd1: %f  bnd2: %f\n", 
+			  buff, read_sense1, read_sense2, bnd1, bnd2);
+		  throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
 		}
 		collow[icol] = bnd1;
 		colup[icol] = bnd1;
@@ -1710,12 +1809,15 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 	icol = findHash(buff, 1);
 
 #ifdef LPIO_DEBUG
-	printf("CoinLpIO::readLp(): integer: colname: (%s)  icol: %d\n", 
+	printf("CoinLpIO::readLp(): Integer: colname: (%s)  icol: %d\n", 
 	       buff, icol);
 #endif
 
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Integer variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Integer variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1723,7 +1825,7 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 	  }
 	  
 #ifdef LPIO_DEBUG
-	  printf("CoinLpIO::readLp(): integer: colname: (%s)  icol: %d\n", 
+	  printf("CoinLpIO::readLp(): Integer: colname: (%s)  icol: %d\n", 
 		 buff, icol);
 #endif
 	  
@@ -1748,7 +1850,10 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 #endif
 
 	if(icol < 0) {
-	  printf("### WARNING: CoinLpIO::readLp(): Binary variable %s does not appear in objective function or constraints\n", buff);
+	  char printBuffer[512];
+	  sprintf(printBuffer,"### CoinLpIO::readLp(): Binary variable %s does not appear in objective function or constraints", buff);
+	  handler_->message(COIN_GENERAL_WARNING,messages_)<<printBuffer
+							   <<CoinMessageEol;
 	  insertHash(buff, 1);
 	  icol = findHash(buff, 1);
 	  if(icol == maxcol) {
@@ -1776,9 +1881,9 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
     case 4: done = 1; break;
       
     default: 
-      printf("### ERROR: CoinLpIO::readLp(): Lost while reading: (%s)\n",
-	     buff);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Lost while reading: (%s)\n", buff);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
       break;
     }
   }
@@ -1787,7 +1892,7 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
   printf("CoinLpIO::readLp(): Done with reading the Lp file\n");
 #endif
 
-  int *ind = (int *) malloc ((maxcoeff+1) * sizeof(int));
+  int *ind = reinterpret_cast<int *> (malloc ((maxcoeff+1) * sizeof(int)));
 
   for(i=0; i<cnt_coeff; i++) {
     ind[i] = findHash(colNames[i], 1);
@@ -1798,28 +1903,32 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 #endif
 
     if(ind[i] < 0) {
-      printf("### ERROR: CoinLpIO::readLp(): Problem with hash tables\nCan not find name %s\n", colNames[i]);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Hash table: %s not found\n", colNames[i]);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
   }
   
   numberColumns_ = numberHash_[1];
   numberElements_ = cnt_coeff - start[0];
 
-  double *obj = (double *) malloc (numberColumns_ * sizeof(double));
+  double *obj = reinterpret_cast<double *> (malloc (numberColumns_ * sizeof(double)));
   memset(obj, 0, numberColumns_ * sizeof(double));
 
   for(i=0; i<cnt_obj; i++) {
     icol = findHash(colNames[i], 1);
     if(icol < 0) {
-      printf("### ERROR: CoinLpIO::readLp(): Problem with hash tables\nCan not find name %s (objective function)\n", colNames[i]);
-      exit(1);
+      char str[8192];
+      sprintf(str,"### ERROR: Hash table: %s (obj) not found\n", colNames[i]);
+      throw CoinError(str, "readLp", "CoinLpIO", __FILE__, __LINE__);
     }
     obj[icol] = objsense * coeff[i];
   }
 
   if (objsense == -1) {
-    printf("### WARNING: CoinLpIO::readLp(): Maximization problem reformulated as minimization\n");
+    handler_->message(COIN_GENERAL_INFO,messages_)<<
+      " CoinLpIO::readLp(): Maximization problem reformulated as minimization"
+						  <<CoinMessageEol;
     objectiveOffset_ = -objectiveOffset_;
   }
 
@@ -1853,7 +1962,9 @@ variable: %s read_sense1: %d  read_sense2: %d\n", buff, read_sense1, read_sense2
 
   if(are_invalid_names(names_[1], numberHash_[1], false)) {
     setDefaultColNames();
-    printf("### WARNING: CoinLpIO::readLp(): Invalid column names\nNow using default column names.\n");
+    handler_->message(COIN_GENERAL_WARNING,messages_)<<
+      "### CoinLpIO::readLp(): Invalid column names\nNow using default column names."
+						     <<CoinMessageEol;
   } 
   
   for(i=0; i<cnt_coeff; i++) {
@@ -1950,10 +2061,8 @@ CoinLpIO::print() const {
 /*************************************************************************/
 // Hash functions slightly modified from CoinMpsIO.cpp
 
-static int
-hash(const char *name, int maxsiz, int length)
-{
-  static int mmult[] = {
+namespace {
+  const int mmult[] = {
     262139, 259459, 256889, 254291, 251701, 249133, 246709, 244247,
     241667, 239179, 236609, 233983, 231289, 228859, 226357, 223829,
     221281, 218849, 216319, 213721, 211093, 208673, 206263, 203773,
@@ -1965,6 +2074,9 @@ hash(const char *name, int maxsiz, int length)
     103387, 101021, 98639, 96179, 93911, 91583, 89317, 86939, 84521,
     82183, 79939, 77587, 75307, 72959, 70793, 68447, 66103
   };
+ int compute_hash(const char *name, int maxsiz, int length)
+{
+  
   int n = 0;
   int j;
 
@@ -1975,6 +2087,7 @@ hash(const char *name, int maxsiz, int length)
   }
   return ( abs ( n ) % maxsiz );	/* integer abs */
 }
+} // end file-local namespace
 
 /************************************************************************/
 //  startHash.  Creates hash list for names
@@ -1989,7 +2102,7 @@ CoinLpIO::startHash(char const * const * const names,
   int maxhash = maxHash_[section];
   COINColumnIndex i, ipos, iput;
 
-  names_[section] = (char **) malloc(maxhash * sizeof(char *));
+  names_[section] = reinterpret_cast<char **> (malloc(maxhash * sizeof(char *)));
   hash_[section] = new CoinHashLink[maxhash];
   
   CoinHashLink * hashThis = hash_[section];
@@ -2008,9 +2121,9 @@ CoinLpIO::startHash(char const * const * const names,
   
   for (i=0; i<number; i++) {
     const char *thisName = names[i];
-    int length = strlen(thisName);
+    int length = CoinStrlenAsInt(thisName);
     
-    ipos = hash(thisName, maxhash, length);
+    ipos = compute_hash(thisName, maxhash, length);
     if (hashThis[ipos].index == -1) {
       hashThis[ipos].index = i; // will be changed below
     }
@@ -2029,9 +2142,9 @@ CoinLpIO::startHash(char const * const * const names,
   iput = -1;
   for (i=0; i<number; i++) {
     const char *thisName = names[i];
-    int length = strlen(thisName);
+    int length = CoinStrlenAsInt(thisName);
     
-    ipos = hash(thisName, maxhash, length);
+    ipos = compute_hash(thisName, maxhash, length);
     
     while (1) {
       COINColumnIndex j1 = hashThis[ipos].index;
@@ -2049,9 +2162,9 @@ CoinLpIO::startHash(char const * const * const names,
 
 #ifdef LPIO_DEBUG
 	if(j1 > i) {
-	  printf("### ERROR: CoinLpIO::startHash(): j1: %d  i: %d\n",
-		 j1, i);
-	  exit(1);
+	  char str[8192];
+	  sprintf(str,"### ERROR: Hash table: j1: %d  i: %d\n", j1, i);
+	  throw CoinError(str, "startHash", "CoinLpIO", __FILE__, __LINE__);
 	}
 #endif
 
@@ -2073,8 +2186,9 @@ CoinLpIO::startHash(char const * const * const names,
 	    while (1) {
 	      ++iput;
 	      if (iput > maxhash) {
-		printf ( "### ERROR: CoinLpIO::startHash(): too many names\n");
-		exit(1);
+		char str[8192];
+		sprintf(str,"### ERROR: Hash table: too many names\n");
+		throw CoinError(str, "startHash", "CoinLpIO", __FILE__, __LINE__);
 		break;
 	      }
 	      if (hashThis[iput].index == -1) {
@@ -2143,9 +2257,9 @@ CoinLpIO::findHash(const char *name, int section) const
   if (!maxhash)
     return -1;
 
-  int length = strlen (name);
+  int length = CoinStrlenAsInt(name);
 
-  ipos = hash (name, maxhash, length);
+  ipos = compute_hash(name, maxhash, length);
   while (1) {
     COINColumnIndex j1 = hashThis[ipos].index;
 
@@ -2185,9 +2299,9 @@ CoinLpIO::insertHash(const char *thisName, int section)
   char **hashNames = names_[section];
 
   int iput = -1;
-  int length = strlen (thisName);
+  int length = CoinStrlenAsInt(thisName);
 
-  int ipos = hash (thisName, maxhash, length);
+  int ipos = compute_hash(thisName, maxhash, length);
 
   while (1) {
     COINColumnIndex j1 = hashThis[ipos].index;
@@ -2206,8 +2320,9 @@ CoinLpIO::insertHash(const char *thisName, int section)
 	  while (1) {
 	    ++iput;
 	    if (iput == maxhash) {
-	      printf ( "### ERROR: CoinLpIO::insertHash(): too many names\n");
-	      exit(1);
+	      char str[8192];
+	      sprintf(str,"### ERROR: Hash table: too many names\n");
+	      throw CoinError(str, "insertHash", "CoinLpIO", __FILE__, __LINE__);
 	      break;
 	    }
 	    if (hashThis[iput].index == -1) {
@@ -2230,4 +2345,19 @@ CoinLpIO::insertHash(const char *thisName, int section)
   (numberHash_[section])++;
 
 }
+// Pass in Message handler (not deleted at end)
+void 
+CoinLpIO::passInMessageHandler(CoinMessageHandler * handler)
+{
+  if (defaultHandler_) 
+    delete handler_;
+  defaultHandler_=false;
+  handler_=handler;
+}
+// Set language
+void 
+CoinLpIO::newLanguage(CoinMessages::Language language)
+{
+  messages_ = CoinMessage(language);
+}
 
