@@ -1,4 +1,5 @@
-// Last edit: 2/10/07
+/* $Id: CoinLpIO.hpp 1448 2011-06-19 15:34:41Z stefan $ */
+// Last edit: 11/5/08
 //
 // Name:     CoinLpIO.hpp; Support for Lp files
 // Author:   Francois Margot
@@ -9,10 +10,13 @@
 //-----------------------------------------------------------------------------
 // Copyright (C) 2003, Francois Margot, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 
 #include <cstdio>
 
 class CoinPackedMatrix;
+#include "CoinMessage.hpp"
 
 typedef int COINColumnIndex;
 
@@ -32,6 +36,7 @@ typedef int COINColumnIndex;
  0 <= x1 <= 3 <BR>
  1 >= x2 <BR>
  x3 = 1 <BR>
+ -2 <= x4 <= Inf <BR>
  xyr free <BR>
  Integers <BR>
  x0 <BR>
@@ -43,7 +48,7 @@ typedef int COINColumnIndex;
 
 Notes: <UL>
  <LI> Keywords are: Min, Max, Minimize, Maximize, s.t., Subject To, 
-      Bounds, Integers, Generals, Binaries, End, Free. 
+      Bounds, Integers, Generals, Binaries, End, Free, Inf. 
  <LI> Keywords are not case sensitive and may be in plural or singular form.
       They should not be used as objective, row or column names.
  <LI> Bounds, Integers, Generals, Binaries sections are optional.
@@ -439,11 +444,45 @@ public:
   /// Dump the data. Low level method for debugging.
   void print() const;
   //@}
+/**@name Message handling */
+//@{
+  /** Pass in Message handler
+  
+      Supply a custom message handler. It will not be destroyed when the
+      CoinMpsIO object is destroyed.
+  */
+  void passInMessageHandler(CoinMessageHandler * handler);
+
+  /// Set the language for messages.
+  void newLanguage(CoinMessages::Language language);
+
+  /// Set the language for messages.
+  inline void setLanguage(CoinMessages::Language language) {newLanguage(language);}
+
+  /// Return the message handler
+  inline CoinMessageHandler * messageHandler() const {return handler_;}
+
+  /// Return the messages
+  inline CoinMessages messages() {return messages_;}
+  /// Return the messages pointer
+  inline CoinMessages * messagesPointer() {return & messages_;}
+//@}
 
 protected:
   /// Problem name
   char * problemName_;
 
+  /// Message handler
+  CoinMessageHandler * handler_;
+  /** Flag to say if the message handler is the default handler.
+      
+      If true, the handler will be destroyed when the CoinMpsIO
+      object is destroyed; if false, it will not be destroyed.
+  */
+  bool defaultHandler_;
+  /// Messages
+  CoinMessages messages_;
+
   /// Number of rows
   int numberRows_;
   
@@ -609,6 +648,10 @@ protected:
   /// Return 0 otherwise.
   int is_free(const char *buff) const;
   
+  /// Return 1 if buff is the keyword "inf" or one of its variants.
+  /// Return 0 otherwise.
+  int is_inf(const char *buff) const;
+  
   /// Return an integer indicating the inequality sense read.
   /// Return 0 if buff is '<='.
   /// Return 1 if buff is '='.
