@@ -1,18 +1,23 @@
-/* $Id: CoinPackedVector.hpp 1215 2009-11-05 11:03:04Z forrest $ */
+/* $Id: CoinPackedVector.hpp 1448 2011-06-19 15:34:41Z stefan $ */
 // Copyright (C) 2000, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #ifndef CoinPackedVector_H
 #define CoinPackedVector_H
 
-#if defined(_MSC_VER)
-// Turn off compiler warning about long names
-#  pragma warning(disable:4786)
-#endif
-
 #include <map>
 
+#include "CoinPragma.hpp"
 #include "CoinPackedVectorBase.hpp"
 #include "CoinSort.hpp"
+
+#ifdef COIN_FAST_CODE
+#ifndef COIN_NOTEST_DUPLICATE
+#define COIN_NOTEST_DUPLICATE
+#endif
+#endif
+
 #ifndef COIN_NOTEST_DUPLICATE
 #define COIN_DEFAULT_VALUE_FOR_DUPLICATE true
 #else
